      REAL FUNCTION alog1p(x)
C     (log1p)
C     Return alog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses minimax rational polynomials computed by Maple,
C     e.g.,
C
C         with(numapprox):
C         Digits := 35:
C         minimax((ln(1+x)-x)/x^2, x = -0.5 .. -0.375, [2,2], 1, 'err');
C         printf("%.2e\n", err):
C
C     on 8 subintervals, chosen to have accuracy adequate for IEEE 754
C     and VAX 32-bit arithmetic.
C     (25-Jun-2002)
      REAL x
      REAL ONE, ZERO
C
      DATA ONE   / 1.0e+00 /
      DATA ZERO  / 0.0e+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, -0.5):  log(1+x)
C     x in [-0.5, 0.5]:        minimax rational polynomials in 8 blocks
C     x in (0.5, Infinity]:    log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of log1p(NaN) and log1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          alog1p = x
      ELSE IF (x .eq. ZERO) THEN
          alog1p = x
      ELSE IF (x .lt. -0.5) THEN
          alog1p = alog(ONE + x)
      ELSE IF (x .le. -0.375) THEN
C
C         Error = 1.12e-09
C
          alog1p = x + ( x * x * (
     X        ( -1.10319634e+00 +
     X        ( -8.94165943e-01 +
     X        ( -3.48931302e-02
     X        ) * x) * x)
     X        /
     X        (  2.20619640e+00 +
     X        (  3.25727364e+00 +
     X        (  1.13188626e+00
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. -0.25) THEN
C
C         Error = 3.47e-10
C
          alog1p = x + ( x * x * (
     X        ( -8.32017042e-01 +
     X        ( -6.33426416e-01 +
     X        ( -1.85316714e-02
     X        ) * x) * x)
     X        /
     X        (  1.66401760e+00 +
     X        (  2.37596297e+00 +
     X        (  7.87785527e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. -0.125) THEN
C
C         Error = 1.30e-10
C
          alog1p = x + ( x * x * (
     X        ( -6.61029376e-01 +
     X        ( -4.75789993e-01 +
     X        ( -1.08806554e-02
     X        ) * x) * x)
     X        /
     X        (  1.32205814e+00 +
     X        (  1.83293608e+00 +
     X        (  5.82531387e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.0) THEN
C
C         Error = 5.59e-11
C
          alog1p = x + ( x * x * (
     X        ( -5.44649263e-01 +
     X        ( -3.72368610e-01 +
     X        ( -6.86612228e-03
     X        ) * x) * x)
     X        /
     X        (  1.08929852e+00 +
     X        (  1.47093619e+00 +
     X        (  4.49704447e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.125) THEN
C
C         Error = 2.66e-11
C
          alog1p = x + ( x * x * (
     X        ( -4.60965125e-01 +
     X        ( -3.00442156e-01 +
     X        ( -4.57525128e-03
     X        ) * x) * x)
     X        /
     X        (  9.21930250e-01 +
     X        (  1.21550446e+00 +
     X        (  3.58523131e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.25) THEN
C
C         Error = 1.37e-11
C
          alog1p = x + ( x * x * (
     X        ( -3.98247626e-01 +
     X        ( -2.48181185e-01 +
     X        ( -3.18151103e-03
     X        ) * x) * x)
     X        /
     X        (  7.96495375e-01 +
     X        (  1.02735556e+00 +
     X        (  2.93064970e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.375) THEN
C
C         Error = 7.53e-12
C
          alog1p = x + ( x * x * (
     X        ( -3.49698421e-01 +
     X        ( -2.08891064e-01 +
     X        ( -2.28959247e-03
     X        ) * x) * x)
     X        /
     X        (  6.99398007e-01 +
     X        (  8.84026296e-01 +
     X        (  2.44392417e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.5) THEN
C
C         Error = 4.36e-12
C
          alog1p = x + ( x * x * (
     X        ( -3.11127985e-01 +
     X        ( -1.78532909e-01 +
     X        ( -1.69493026e-03
     X        ) * x) * x)
     X        /
     X        (  6.22260383e-01 +
     X        (  7.71847181e-01 +
     X        (  2.07160762e-01
     X        ) * x) * x) ) )
C
      ELSE
          alog1p = alog(ONE + x)
      END IF
C
      END

      REAL FUNCTION al1p10(x)
C     (log1p10)
C     Return alog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      REAL x
      REAL alog1p
      REAL LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366E+00/
C
      al1p10 = alog1p(x) * LOG10E
C
      END
