      REAL FUNCTION alog1p(x)
C     (log1p)
C     Return alog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses minimax rational polynomials computed by Maple,
C     e.g.,
C
C         with(numapprox):
C         Digits := 35:
C         minimax((log(1+x) - x + x^2/2)/x^3,
C                x = -0.5 .. -0.375, [2,2], 1, 'err');
C         printf("%.2e\n", err):
C
C     on 8 subintervals, chosen to have accuracy adequate for IEEE 754
C     and VAX 32-bit arithmetic.
C     (25-Jun-2002)
      REAL x
      REAL ONE, ZERO
C
      DATA ONE   / 1.0e+00 /
      DATA ZERO  / 0.0e+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, -0.5):  log(1+x)
C     x in [-0.5, 0.5]:        minimax rational polynomials in 8 blocks
C     x in (0.5, Infinity]:    log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of log1p(NaN) and log1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          alog1p = x
      ELSE IF (x .eq. ZERO) THEN
          alog1p = x
      ELSE IF (x .lt. -0.5) THEN
          alog1p = alog(ONE + x)
      ELSE IF (x .le. -0.375) THEN
C
C         Error = 6.05e-10
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  7.68347536e-01 +
     X        (  6.13870275e-01 +
     X        (  1.24874124e-02
     X        ) * x) * x)
     X        /
     X        (  2.30489245e+00 +
     X        (  3.56887723e+00 +
     X        (  1.32650068e+00
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. -0.25) THEN
C
C         Error = 1.76e-10
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  5.72025244e-01 +
     X        (  4.33490959e-01 +
     X        (  6.15881244e-03
     X        ) * x) * x)
     X        /
     X        (  1.71606366e+00 +
     X        (  2.58734997e+00 +
     X        (  9.28458855e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. -0.125) THEN
C
C         Error = 6.26e-11
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  4.48745153e-01 +
     X        (  3.24469606e-01 +
     X        (  3.38820663e-03
     X        ) * x) * x)
     X        /
     X        (  1.34623503e+00 +
     X        (  1.98307385e+00 +
     X        (  6.89618750e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.0) THEN
C
C         Error = 2.56e-11
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  3.65225142e-01 +
     X        (  2.53047526e-01 +
     X        (  2.01694814e-03
     X        ) * x) * x)
     X        /
     X        (  1.09567542e+00 +
     X        (  1.58089912e+00 +
     X        (  5.34318110e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.125) THEN
C
C         Error = 1.16e-11
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  3.05462850e-01 +
     X        (  2.03477922e-01 +
     X        (  1.27461719e-03
     X        ) * x) * x)
     X        /
     X        (  9.16388551e-01 +
     X        (  1.29772517e+00 +
     X        (  4.27285548e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.25) THEN
C
C         Error = 5.74e-12
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  2.60898041e-01 +
     X        (  1.67545983e-01 +
     X        (  8.44225312e-04
     X        ) * x) * x)
     X        /
     X        (  7.82694202e-01 +
     X        (  1.08965620e+00 +
     X        (  3.50188060e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.375) THEN
C
C         Error = 3.03e-12
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  2.26575319e-01 +
     X        (  1.40599008e-01 +
     X        (  5.80770352e-04
     X        ) * x) * x)
     X        /
     X        (  6.79726685e-01 +
     X        (  9.31578715e-01 +
     X        (  2.92692995e-01
     X        ) * x) * x) ) )
C
      ELSE IF (x .le. 0.5) THEN
C
C         Error = 1.69e-12
C
          alog1p = x - 0.5e+00 * x * x + (x * x * x * (
     X        (  1.99445677e-01 +
     X        (  1.19830005e-01 +
     X        (  4.12231312e-04
     X        ) * x) * x)
     X        /
     X        (  5.98339725e-01 +
     X        (  8.08208504e-01 +
     X        (  2.48599554e-01
     X        ) * x) * x) ) )
C
      ELSE
          alog1p = alog(ONE + x)
      END IF
C
      END

      REAL FUNCTION al1p10(x)
C     (log1p10)
C     Return alog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      REAL x
      REAL alog1p
      REAL LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366E+00/
C
      al1p10 = alog1p(x) * LOG10E
C
      END
