      REAL FUNCTION expm1(x)
C     (expm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses minimax rational polynomials computed by Maple,
C     e.g.,
C
C         with(numapprox):
C         Digits := 20:
C         minimax((exp(x)-1-x)/x^2, x = -0.5 .. -0.25, [2,2], 1, 'err');
C         printf("%.2e\n", err):
C
C     on 5 subintervals, chosen to have accuracy adequate for IEEE 754
C     and VAX 32-bit arithmetic.
C     (28-Jun-2002)
      REAL x
      REAL ONE, ZERO
C
      DATA ZERO / 0.0E+00 /
      DATA ONE  / 1.0E+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, -0.75):  exp(x) - 1
C     x in [-0.75, 0.50):       minimax rational polynomials in 5 blocks
C     x in [0.50, Infinity]:    exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .NE. x) THEN
          expm1 = x
      ELSE IF (x .EQ. ZERO) THEN
          expm1 = x
      ELSE IF (x .LT. -0.75) THEN
          expm1 = exp(x) - ONE
      ELSE IF (x .LT. -0.50) THEN
C
C         [-0.75,-0.50) Error = 1.42e-11
C
          expm1 = x + (x * x * (
     X (  4.04382035e-01 +
     X ( -8.21183611e-03 +
     X (  1.67522215e-03
     X ) * x) * x)
     X /
     X (  8.08765766e-01 +
     X ( -2.85997735e-01 +
     X (  3.13365322e-02
     X ) * x) * x) ) )
C
      ELSE IF (x .LT. -0.25) THEN
C
C         [-0.50,-0.25) Error = 1.79e-11
C
          expm1 = x + (x * x * (
     X (  4.40557596e-01 +
     X ( -5.30346777e-03 +
     X (  2.05099129e-03
     X ) * x) * x)
     X /
     X (  8.81115328e-01 +
     X ( -3.04310057e-01 +
     X (  3.21239583e-02
     X ) * x) * x) ) )
C
      ELSE IF (x .LT. 0.0) THEN
C
C         [-0.25,-0.0) Error = 2.25e-11
C
          expm1 = x + (x * x * (
     X (  4.79403173e-01 +
     X ( -1.88783908e-03 +
     X (  2.51101778e-03
     X ) * x) * x)
     X /
     X (  9.58806346e-01 +
     X ( -3.23377784e-01 +
     X (  3.29143962e-02
     X ) * x) * x) ) )
C
      ELSE IF (x .LT. 0.25) THEN
C
C         [0.0,0.25) Error = 2.83e-11
C
          expm1 = x + (x * x * (
     X (  5.21055248e-01 +
     X (  2.08307010e-03 +
     X (  3.07424218e-03
     X ) * x) * x)
     X /
     X (  1.04211050e+00 +
     X ( -3.43204014e-01 +
     X (  3.37069272e-02
     X ) * x) * x) ) )
C
      ELSE IF (x .LT. 0.50) THEN
C
C         [0.25,0.50) Error = 3.56e-11
C
          expm1 = x + (x * x * (
     X (  5.65650116e-01 +
     X (  6.65361369e-03 +
     X (  3.76388598e-03
     X ) * x) * x)
     X /
     X (  1.13130002e+00 +
     X ( -3.63789880e-01 +
     X (  3.45006083e-02
     X ) * x) * x) ) )
      ELSE
          expm1 = exp(x) - ONE
      ENDIF
C
      END
